package cellmodels.simplecell;

import java.util.*;
import core.*;
import util.*;

class SimpleCell extends Cell
{
	Vector sensedChemicals;

	SimpleCell(Integer p_ID, SystemModel p_sys, CellProgram p_cp, Vector args)
	{
		super(p_ID,p_sys,p_cp);
		sensedChemicals = new Vector();
	}

	public void init(Vector args)
	{
		if(args != null)
		{
			setState((String)args.elementAt(0));
		}
	}

	public Vector provideService(Vector args)
	{
		Vector retVec = new Vector();
		if(args == null)
			return null;

		String service = (String)args.elementAt(0);

		if(service.equals(new String("Sense Chemicals")))
		{
			sensedChemicals = (Vector)args.elementAt(1);

			return null;
		}
		else if(service.equals(new String("Emit Chemical")))
		{
			Vector emissions = (Vector)args.elementAt(1);
			Vector argVector = new Vector();
			argVector.add(new String("Diffuse Chemical"));
			argVector.add(new Integer(getID().intValue()));
			argVector.add(emissions);
			getSystem().provideService(argVector);

			return null;
		}
		else if(service.equals(new String("Check Condition")))
		{
			ChemicalCondition chemCondition = (ChemicalCondition)args.elementAt(1);
			Enumeration enum = sensedChemicals.elements();
			ChemicalConcentration chem;

			while(enum.hasMoreElements())
			{
				chem = (ChemicalConcentration)enum.nextElement();
				int comparison = chemCondition.compareTo(chem);

				if( comparison == 0)
				{
					retVec.add(new Boolean(true));
					return retVec;
				}
				if( (comparison == 1) || (comparison == -1) )
				{
					retVec.add(new Boolean(false));
					return retVec;
				}
			}

			/* Chemical not found so check again with concentration 0.0 */
			chem = new ChemicalConcentration(chemCondition.getChemical(),0.0f);
			if( chemCondition.compareTo(chem) == 0)
			{
				retVec.add(new Boolean(true));
				return retVec;
			}
			else
			{
				retVec.add(new Boolean(false));
				return retVec;
			}
		}
		else if(service.equals(new String("State Change")))
		{
			setState((String)args.elementAt(1));

			return null;
		}
		else if(service.equals(new String("Cell Division")))
		{
			String state1 = (String)args.elementAt(1);
			String state2 = (String)args.elementAt(2);
			String divisionType = (String)args.elementAt(3);

			Vector argVector = new Vector();
			argVector.add(new String("Cell Division"));
			argVector.add(new Integer(getID().intValue()));
			argVector.add(state1);
			argVector.add(state2);
			argVector.add(divisionType);
			getSystem().provideService(argVector);

			return null;
		}
		return null;
	}

	public Color getColor()
	{
		Vector args = new Vector();
		args.add(new String("Get State Color"));
		args.add(new String(getState()));
		Vector retVec = getCellProgram().provideService(args);
		return (Color)retVec.elementAt(0);
	}

	static class SimpleCellFactory extends CellFactory
	{
		public Cell create(Integer p_ID, SystemModel p_sys, CellProgram p_cp, String modelOptions, Vector args)
		{
			Cell cell = new SimpleCell(p_ID, p_sys, p_cp, args);
			cell.init(args);
			return cell;
		}
	}

	static
	{
		Simulation.cellModels.put(new String("cellmodels.simplecell.SimpleCell"), new SimpleCellFactory());
	}
}
