package core;

import java.util.*;
import java.io.*;
import javax.media.j3d.*;

import core.*;

public class Simulation
{
	public static Hashtable sysModels = new Hashtable();
	public static Hashtable cellModels = new Hashtable();
	public static Hashtable parsers = new Hashtable();

	Properties simFile = new Properties();
	BufferedReader cellProgramReader;
	BufferedReader simFileReader;
	BufferedWriter outputWriter;

	String systemModelName;
	String systemModelOptions;
	String cellModelName;
	String cellModelOptions;
	String parserModelName;
	String parserModelOptions;

	SystemModel sys;
	CellProgram cellProgram;

	int step = 0;

	public Simulation(Vector ioObjects)
	{
		simFileReader = (BufferedReader)ioObjects.elementAt(0);
		cellProgramReader = (BufferedReader)ioObjects.elementAt(1);
		outputWriter = (BufferedWriter)ioObjects.elementAt(2);

		try
		{
			String s = simFileReader.readLine();
			while(s != null)
			{
				if(s.substring(0,7).equals("system="))
				{
					simFile.setProperty("system", s.substring(7));
				}
				else if(s.substring(0,5).equals("cell="))
				{
					simFile.setProperty("cell", s.substring(5));
				}
				else if(s.substring(0,7).equals("parser="))
				{
					simFile.setProperty("parser", s.substring(7));
				}
				s = simFileReader.readLine();
			}
		}
		catch(IOException e)
		{
			System.err.println("Io Exception ocurred : " + e.toString());
			System.exit(0);
		}
		systemModelName = new String(simFile.getProperty("system"));
		systemModelOptions = new String("");
		cellModelName = new String(simFile.getProperty("cell"));
		cellModelOptions = new String("");
		parserModelName = new String(simFile.getProperty("parser"));
		parserModelOptions = new String("");

		try
		{
			cellProgram = createCellProgram(parserModelName, parserModelOptions, cellProgramReader);
			sys = createSystem(systemModelName, systemModelOptions);
		}
		catch(Exception e)
		{
			System.err.println(e);
			System.exit(0);
		}
	}

	public BranchGroup getDisplayGraph()
	{
		return sys.getDisplayGraph();
	}

	public CellProgram getCellProgram()
	{
		return cellProgram;
	}

	public void nextStep()
	{
		step++;
		sys.runCellProgramForOneStep();
	}

	public int getStep()
	{
		return step;
	}

	public Cell createCell(Integer p_ID, SystemModel p_sys, CellProgram p_cp, Vector args) throws Exception
	{
		CellFactory s = (CellFactory)cellModels.get(cellModelName);
		if(s == null) // shape not found
		{
			try
			{
				Class.forName(cellModelName);
				// By now the static block with no function would
				// have been executed if the system was found.
				// the system is expected to have put its factory
				// in the hashtable.
				s = (CellFactory) cellModels.get(cellModelName);
				if(s == null) // if the system factory is not there even now
				{
					throw (new Exception("System not created"));
				}
			}
			catch(ClassNotFoundException e)
			{
				// We'll throw an exception to indicate that
				// the shape could not be created
					throw (new Exception("System not created"));
			}
		}
		return s.create(p_ID, p_sys, p_cp, cellModelOptions, args);
	}

	public CellProgram createCellProgram(String modelName, String modelOptions, BufferedReader inp)
							throws GenericParseException, Exception
	{
		ParserFactory s = (ParserFactory)parsers.get(modelName);
		if(s == null) // shape not found
		{
			try
			{
				Class.forName(modelName);
				// By now the static block with no function would
				// have been executed if the Parser was found.
				// the Parser is expected to have put its factory
				// in the hashtable.
				s = (ParserFactory) parsers.get(modelName);
				if(s == null) // if the system factory is not there even now
				{
					throw (new Exception("Parser not created"));
				}
			}
			catch(ClassNotFoundException e)
			{
				// We'll throw an exception to indicate that
				// the shape could not be created
					throw (new Exception("Parser not created"));
			}
		}
		Parser p = s.create(inp,modelOptions);
		CellProgram cp;
		try
		{
			cp = p.getCellProgram();
		}
		catch(core.GenericParseException e)
		{
			throw new GenericParseException(e.toString());
		}
		return cp;
	}

	public SystemModel createSystem(String modelName, String modelOptions) throws Exception
	{
		SystemFactory s = (SystemFactory)sysModels.get(modelName);
		if(s == null) // shape not found
		{
			try
			{
				Class.forName(modelName);
				// By now the static block with no function would
				// have been executed if the System was found.
				// the System is expected to have put its factory
				// in the hashtable.
				s = (SystemFactory) sysModels.get(modelName);
				if(s == null) // if the system factory is not there even now
				{
					throw (new Exception("System not created"));
				}
			}
			catch(ClassNotFoundException e)
			{
				// We'll throw an exception to indicate that
				// the shape could not be created
					throw (new Exception("System not created"));
			}
		}
		return s.create(this,modelOptions);
	}
}
