package gui;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import com.sun.j3d.utils.geometry.ColorCube;
import com.sun.j3d.utils.universe.*;
import javax.media.j3d.*;
import javax.vecmath.*;

import gui.SimWinMgr;
import core.Simulation;

class SimulationWindow extends JInternalFrame implements InternalFrameListener
{
	SimWinMgr simWinMgr;
	Simulation sim;
	Integer windowID;

	/* 3D Display objects */
    TransformGroup objTrans;
    Transform3D trans = new Transform3D();
    JButton buttonRotate = new JButton("Rotate");
    private SimpleUniverse universe = null;
	BranchGroup scene;
	BranchGroup objRoot;
	BranchGroup lightsGroup;

	SimulationWindow(Integer p_windowID, String title, Rectangle bounds, SimWinMgr p_simWinMgr, Vector ioObjects)
	{
		windowID = new Integer(p_windowID.intValue());
		simWinMgr = p_simWinMgr;
		sim = new Simulation(ioObjects);

		setBounds(bounds);
		setIconifiable(true);
		setResizable(true);
		setMaximizable(true);
		setClosable(true);
		setTitle(windowID + ". " + title);
		addInternalFrameListener(this);

		Container mainContainer = getContentPane();
		mainContainer.setLayout(new BorderLayout());
        GraphicsConfiguration config = SimpleUniverse.getPreferredConfiguration();

        Canvas3D c = new Canvas3D(config);
		mainContainer.add("Center", c);
		JPanel p = new JPanel();
		p.add(buttonRotate);
		mainContainer.add("North", p);

		// Create a simple scene and attach it to the virtual universe
		lightsGroup = null;
		objRoot = null;
		universe = new SimpleUniverse(c);
		universe.getViewingPlatform().setNominalViewingTransform();
		scene = new BranchGroup();
		scene.setCapability( BranchGroup.ALLOW_BOUNDS_READ );
		scene.setCapability( BranchGroup.ALLOW_CHILDREN_WRITE );
		scene.setCapability( BranchGroup.ALLOW_CHILDREN_READ );
		scene.setCapability( BranchGroup.ALLOW_CHILDREN_EXTEND );
		createSceneGraph();
		universe.addBranchGraph(scene);
	}

	/* Implementing the InternalFrameListener */
	public void internalFrameClosing(InternalFrameEvent e) {
		simWinMgr.deleteSimulationWindow(windowID);
    }

    public void internalFrameClosed(InternalFrameEvent e) {
    }

    public void internalFrameOpened(InternalFrameEvent e) {
    }

    public void internalFrameIconified(InternalFrameEvent e) {
    }

    public void internalFrameDeiconified(InternalFrameEvent e) {
    }

    public void internalFrameActivated(InternalFrameEvent e) {
		simWinMgr.setCurrentSimulationWindow(windowID);
    }

    public void internalFrameDeactivated(InternalFrameEvent e) {
    }

    public void createSceneGraph() {
		if(objRoot != null)
		{
			scene.removeChild(scene.indexOfChild(objRoot));
			objRoot = null;
		}

		// Create the root of the branch graph
		objRoot = new BranchGroup();
		objRoot.setCapability( BranchGroup.ALLOW_DETACH );

		// Create the transform group node and initialize it to the
		// identity.  Enable the TRANSFORM_WRITE capability so that
		// our behavior code can modify it at runtime.  Add it to the
		// root of the subgraph.
		objTrans = new TransformGroup();
		objTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		objRoot.addChild(objTrans);

		TransformGroup objScale = new TransformGroup();
		Transform3D t = new Transform3D();
		t.setScale(0.5);
		objScale.setTransform(t);

		// System creates th scene graph
		BranchGroup simGraph = sim.getDisplayGraph();
		objScale.addChild(simGraph);
		objTrans.addChild(objScale);

		// create the AWTInteractionBehavior	
		AWTInteractionBehavior awtBehavior = new AWTInteractionBehavior(objTrans);
		buttonRotate.addActionListener(awtBehavior);

		// Create a bounds for the background and behaviors
		BoundingSphere bounds = new BoundingSphere(new Point3d(0.0, 0.0, 0.0), 100.0);
		awtBehavior.setSchedulingBounds(bounds);
		objRoot.addChild(awtBehavior);

		scene.addChild(objRoot);

		if(lightsGroup == null)
		{
			lightsGroup = new BranchGroup();
			// Set up the background
			Color3f bgColor = new Color3f(0.05f, 0.05f, 0.2f);
			Background bg = new Background(bgColor);
			bg.setApplicationBounds(bounds);
			lightsGroup.addChild(bg);

			// Set up the global lights
			Color3f lColor1 = new Color3f(0.7f, 0.7f, 0.7f);
			Vector3f lDir1  = new Vector3f(-1.0f, -1.0f, -1.0f);
			Color3f alColor = new Color3f(0.2f, 0.2f, 0.2f);

			AmbientLight aLgt = new AmbientLight(alColor);
			aLgt.setInfluencingBounds(bounds);
			DirectionalLight lgt1 = new DirectionalLight(lColor1, lDir1);
			lgt1.setInfluencingBounds(bounds);
			lightsGroup.addChild(aLgt);
			lightsGroup.addChild(lgt1);
			scene.addChild(lightsGroup);
		}
    }

	void nextStep()
	{
		sim.nextStep();
		createSceneGraph();
	}

    public void destroy()
	{
		universe.removeAllLocales();
    }
}
